using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace HabboTraxDEBESTE2
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Player : Microsoft.Xna.Framework.Game
    {
        private Dictionary<string, SoundEffect> soundsLoaded;
        private Trax[] traxNeeded;
        private List<SoundEffectInstance> activeSoundEffects;

        //private Trax testTrax1;
        //private Trax testTrax2;

        public Trax[] TraxNeeded
        {
            get { return traxNeeded; }
            set
            {
                traxNeeded = value.Distinct().ToArray();
                LoadContent();
            }
        }

        public void AddTraxNeeded(Trax[] traxToAdd)
        {
            List<Trax> traxNeededUpdate = TraxNeeded.ToList();
            foreach (Trax trax in traxToAdd)
            {
                if (!traxNeededUpdate.Contains(trax))
                    traxNeededUpdate.Add(trax);
            }
            TraxNeeded = traxNeededUpdate.ToArray();

        }

        public Player()
        {
            //graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            soundsLoaded = new Dictionary<string, SoundEffect>();
            activeSoundEffects = new List<SoundEffectInstance>();
            //testTrax1 = new Trax("01", 1);
            //testTrax2 = new Trax("01", 2);
            TraxNeeded = new Trax[0];
            //{
            //    testTrax1,
            //    testTrax2
            //};
            //LoadContent();
        }

        protected override void LoadContent()
        {
            soundsLoaded.Clear();
            foreach (Trax trax in TraxNeeded)
            {
                try
                {
                    SoundEffect sound = Content.Load<SoundEffect>("Trax\\" + trax.Code);
                    soundsLoaded.Add(trax.Code, sound);
                    trax.Length = sound.Duration.TotalSeconds;
                }
                catch
                {
                    //Trax niet geladen
                }
            }
            base.LoadContent();
        }

        //public void PlaySound()
        //{
        //    soundsLoaded[testTrax1].Play();
        //}

        //public void PlaySound2()
        //{
        //    soundsLoaded[testTrax2].Play();
        //}

        public void PlaySound(Trax trax)
        {
            if (trax == null || !soundsLoaded.ContainsKey(trax.Code))
                return;

            SoundEffect e = soundsLoaded[trax.Code];
            var effectInstance = e.CreateInstance();
            effectInstance.Play();
            System.Threading.ThreadPool.QueueUserWorkItem(delegate
            {
                ClearSoundEffectInstances();
            });
            activeSoundEffects.Add(effectInstance);
        }

        private void ClearSoundEffectInstances()
        {
            try
            {
                List<SoundEffectInstance> remainingSoundEffects = new List<SoundEffectInstance>();
                lock (activeSoundEffects)
                {
                    foreach (SoundEffectInstance sei in activeSoundEffects)
                        if (sei.State == SoundState.Playing)
                            remainingSoundEffects.Add(sei);
                }
                activeSoundEffects = remainingSoundEffects;
            }
            catch
            {
            }
        }

        public void ShutUp()
        {
            foreach (SoundEffectInstance sei in activeSoundEffects)
                sei.Stop(true);
            activeSoundEffects.Clear();
        }

        public double GetLength(Trax trax)
        {
            if (!soundsLoaded.ContainsKey(trax.Code))
                throw new ArgumentException("Gekozen trax is niet ingeladen!");

            return soundsLoaded[trax.Code].Duration.TotalSeconds;
        }

        #region DefaultGameCode

        ///// <summary>
        ///// Allows the game to perform any initialization it needs to before starting to run.
        ///// This is where it can query for any required services and load any non-graphic
        ///// related content.  Calling base.Initialize will enumerate through any components
        ///// and initialize them as well.
        ///// </summary>
        //protected override void Initialize()
        //{
        //    // TODO: Add your initialization logic here

        //    base.Initialize();
        //}

        ///// <summary>
        ///// LoadContent will be called once per game and is the place to load
        ///// all of your content.
        ///// </summary>
        //protected override void LoadContent()
        //{
        //    // Create a new SpriteBatch, which can be used to draw textures.
        //    //spriteBatch = new SpriteBatch(GraphicsDevice);

        //    // TODO: use this.Content to load your game content here
        //}

        ///// <summary>
        ///// UnloadContent will be called once per game and is the place to unload
        ///// all content.
        ///// </summary>
        //protected override void UnloadContent()
        //{
        //    // TODO: Unload any non ContentManager content here
        //}

        ///// <summary>
        ///// Allows the game to run logic such as updating the world,
        ///// checking for collisions, gathering input, and playing audio.
        ///// </summary>
        ///// <param name="gameTime">Provides a snapshot of timing values.</param>
        //protected override void Update(GameTime gameTime)
        //{
        //    // Allows the game to exit
        //    if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        //        this.Exit();

        //    // TODO: Add your update logic here

        //    base.Update(gameTime);
        //}

        ///// <summary>
        ///// This is called when the game should draw itself.
        ///// </summary>
        ///// <param name="gameTime">Provides a snapshot of timing values.</param>
        //protected override void Draw(GameTime gameTime)
        //{
        //    GraphicsDevice.Clear(Color.CornflowerBlue);

        //    // TODO: Add your drawing code here

        //    base.Draw(gameTime);
        //}
        #endregion
    }
}
